#!/usr/bin/env python

import os
import re
import sys
import time

# Debug stuff so I don't have to comment out debug prints all the time
DEBUG = False 
DEBUG_LEVEL = 1

def log(message):
    if DEBUG:
        print(message + "\n")

def log2(message):
    if DEBUG and DEBUG_LEVEL == 2:
        print(message + "\n")

# Config 
MAX_DAYS_FOR_FULL_BACKUP = 60
MAX_DAYS_FOR_INCREMENTAL_BACKUP = 7

def get_most_recent_job_ids(all_jobs, active_jobs, _wanted_job_level):
    """ Returns a list of the most recent date for all active jobs for a client with the wanted job level.
    It's parsed from llist jobs=client|bconsole command.
    """
    # (job_name, job_id)
    # e.g. name.job1, name.job2 etc.
    latest_jobs = {}

    max_days_for_job = -1
    if _wanted_job_level == 'incremental':
        wanted_job_level = 'I'
        max_days_for_job = MAX_DAYS_FOR_INCREMENTAL_BACKUP
    else:
        wanted_job_level = 'F'
        max_days_for_job = MAX_DAYS_FOR_FULL_BACKUP

    for line in all_jobs[7:]:
        if line[0].strip() == '|':
            splits = line[1:].split('|')

 
	    job_id = splits[0].strip().replace(',', '')
	    job_name = splits[1].strip()
            job_startdate = splits[2].strip()
            job_type = splits[3]
            job_level = splits[4].strip()
            job_files = splits[5].strip()
            job_status = splits[7]

            # if job type is B - backup and has status T - terminated and some files were backed up
            if 'B' in job_type and 'T' in job_status and job_files != '0':
                if job_level == wanted_job_level:
                    latest_jobs[job_name] = job_id
   
    log2("Latest jobs: \n{}".format(latest_jobs))

    result = {}
    for k, v in latest_jobs.items():
        if k in active_jobs:
            result[k] = v
        else:
            log("Removing {} from jobs to test - job is no longer active".format(k))

    log2("Latest *active* jobs: \n{}".format(result))

    return result.values();

end_date_regex = re.compile('\\bendtime: [\\d\\-: ]*')
def get_job_endtime(job):
    """ Parses endtime from job info if the job has finished and returns it,
    None otherwise.
    """
    result = end_date_regex.findall(job)
    if len(result) > 0:
        return result[0].replace('endtime: ', '')

    # job hasn't finished yet
    return None

def date_is_older_than(job_date_string, days):
    """ Checks if the 'job_date_string' is older than 'days' days
    and returns true if it is, false otherwise.
    """
    job_date = time.strptime(job_date_string, "%Y-%m-%d %H:%M:%S")
    days_ago = time.gmtime(time.time() - 60*60*24*days)
    return job_date < days_ago

def get_oldest_job_date(job_dates):
    """ Returns the oldest date from a list of dates in a string format.
    """
    dates_as_timestamps = []
    for job_date in job_dates:
        timestamp = time.mktime(time.strptime(job_date, "%Y-%m-%d %H:%M:%S"))    
        dates_as_timestamps.append(timestamp)

    return min(dates_as_timestamps)


def get_active_jobs_for_client(client_name):
    """ Returns a list of all active jobs for the client 'client_name'.
    It is parsed from the config file, because you can't list all available jobs in the bconsole.
    It also finds the correct config file first, because the name of the config file can be different
    from the client name.
    """
    config_contents = ""

    # Find the right config file
    client_name_re = re.compile('Client.*{\\s*Name\\s*=\\s*([\\w\\-\\.]+)')
    root = '/etc/bacula/client.d'
    for item in os.listdir(root):
        if item.endswith('.conf'):
            path = os.path.join(root, item)
            with open(path, 'r') as conf:
                contents = conf.read() 
                name = client_name_re.search(contents).group(1)
                if name == client_name:
                    config_contents = contents
                    break

    # Get the job names from the file
    job_name_re = re.compile('Job\\s*{\\s*Name\\s*=\\s*\\"([\\w\\-\\.]+)')
    job_names = job_name_re.findall(config_contents)
    return job_names

usage_string = "Usage: bacula_checkuser <user> <backup-type>"

def main():
    if len(sys.argv) != 3:
        print(usage_string)
        sys.exit(1)

    backup_level = sys.argv[2]
    if backup_level not in ['incremental', 'full']:
        print(usage_string)
        print("<backup-type> must be either 'incremental' or 'full'")
        sys.exit(1)

    client_name = sys.argv[1]
    log("Client name: {}".format(client_name))

    active_jobs = get_active_jobs_for_client(client_name)
    log("Active client jobs: {}".format(active_jobs))

    all_client_jobs = os.popen('echo \"list jobs client={}\"|bconsole'.format(client_name), 'r').readlines()

    job_ids = get_most_recent_job_ids(all_client_jobs, active_jobs, backup_level)
    log("Most recent job ids to test: {}".format(job_ids))
    
    if len(job_ids) != 0:
        max_days_for_given_level = MAX_DAYS_FOR_FULL_BACKUP if backup_level == 'full' else MAX_DAYS_FOR_INCREMENTAL_BACKUP
        job_endtimes = []
	for job_id in job_ids:
            job_info = os.popen('echo \"llist jobid={}\"|bconsole'.format(job_id), 'r').read()
            job_endtime = get_job_endtime(job_info)
            if job_endtime == None: # job hasn't finished 
                continue

            job_endtimes.append(job_endtime)
     
        oldest_job_date = get_oldest_job_date(job_endtimes)
        print("{}".format(oldest_job_date))
        return
        
    print(0)
    return

if __name__ == '__main__':
    main()

